#!/usr/bin/python

"""
Copyright (c) 2004, Alastair Tse <alastair@tse.id.au>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution.

Neither the name of Alastair Tse nor the names of its contributors may be 
used to endorse or promote products derived from this software without specific 
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN  IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
__author__ = "Alastair Tse <alastair@tse.id.au>"
__license = "BSD"
__url__ = "http://www.liquidx.net/logcaster/"
__version__ = "0.1"
__copyright__ = "Copyright (c) 2005, Alastair Tse" 

import socket
import sys
import select
import traceback

DEFAULT_MULTICAST_GROUP = '239.31.33.7'
DEFAULT_MULTICAST_PORT = 31337
ERR_INVALID_ARGS = 0

def usage(error = ""):
    print "usage: %s [<multicast address> <port>]"
    print ""
    print "example: logrecv.py 239.31.33.7 31337"

def mcast_open(addr, port):
    intf = socket.gethostbyname(socket.gethostname())
    msocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    
    try:
        msocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        msocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
    except:
        pass
        
    msocket.setsockopt(socket.SOL_IP, socket.IP_MULTICAST_TTL, 255)
    msocket.setsockopt(socket.SOL_IP, socket.IP_MULTICAST_LOOP, 1)
    
    try:
        msocket.bind((addr, port))
    except:
        pass
        
    intf_addr = socket.inet_aton(intf) + socket.inet_aton('0.0.0.0')
    memb_addr = socket.inet_aton(addr) + socket.inet_aton('0.0.0.0')
    msocket.setsockopt(socket.SOL_IP, socket.IP_MULTICAST_IF, intf_addr)
    msocket.setsockopt(socket.SOL_IP, socket.IP_ADD_MEMBERSHIP, memb_addr)

    return msocket
    
def mcast_close(msocket, addr):
    # cleanup
    intf = socket.gethostbyname(socket.gethostname())    
    memb_addr = socket.inet_aton(addr) + socket.inet_aton('0.0.0.0')    
    msocket.setsockopt(socket.SOL_IP, socket.IP_DROP_MEMBERSHIP, memb_addr)
    msocket.close()

def main(args):
    # argument parse
    addr = DEFAULT_MULTICAST_GROUP
    port = DEFAULT_MULTICAST_PORT
    
    if len(args) > 0:
	addr = args[0]
	if len(args) > 1:
	    try:
		port = int(args[1])
		if port > 65355:
		    return ERR_INVALID_ARGS
	    except:
		return ERR_INVALID_ARGS

    msocket = mcast_open(addr, port)

    try:
        msocket_open = True
        while msocket_open:
            infd, outfd, exfd = select.select([msocket, sys.stdin], [], [])
            if msocket in infd:
                recvdata, fromaddr = msocket.recvfrom(65535)
                if len(recvdata) == 0:
                    msocket_open = False
                    break
                else:
                    sys.stdout.write(recvdata)
            
            if sys.stdin in infd:
                if len(sys.stdin.readline()) == 0:
                    msocket_open = False
                    break
    except:
        traceback.print_exc()
        
    mcast_close(msocket, addr)
    
if __name__ == "__main__":
    retval = main(sys.argv[1:])
    if retval == ERR_INVALID_ARGS:
        usage(error = "Invalid Arguments")
        
